/*
Copyright Ideaconsult (C) 2005-2007
Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/**
 *
 */
package chemCast.application;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.border.BevelBorder;

import ambit.ui.AboutAction;
import ambit.ui.CoreApp;
import ambit.ui.UITools;
import ambit.ui.actions.ActionFactory;
import chemCast.data.RandomAccessSubsetTableModel;
import chemCast.similarity.data.IChemCastObject;
import chemCast.ui.MoleculesTable;
import chemCast.ui.VisualizationPanel;
import chemCast.ui.actions.ConfigAction;
import chemCast.ui.actions.FileBatchProcessing;

/**
 * @author Nina Jeliazkova
 *
 */
public class ToxmatchApplication extends CoreApp {
	protected ChemCASTModule chemCastData;

    protected java.util.Hashtable<String ,ActionMap> actionsTrain;
    protected java.util.Hashtable<String ,ActionMap> actionsTest;
    ActionMap allActions;

	/**
	 * @param title
	 * @param width
	 * @param height
	 */

	public ToxmatchApplication(String title, int width, int height,String[] args) {
		super(title, width, height,args);
        int state = mainFrame.getExtendedState();

        // Set the maximized bits
        state |= Frame.MAXIMIZED_BOTH;

        // Maximize the frame
        mainFrame.setExtendedState(state);
	}

	/* (non-Javadoc)
	 * @see ambit.ui.CoreApp#initSharedData()
	 */
	protected void initSharedData(String[] args) {
		chemCastData = new ChemCASTModule();
        createActions();
    }
    protected void createActions() {
        allActions = new ActionMap();
        actionsTrain = new Hashtable<String, ActionMap>();
        ActionMap a = new ActionMap();
        actionsTrain.put("File", a);
        chemCastData.createTrainingDatasetActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);
        a = new ActionMap();
        actionsTrain.put("View", a);
        chemCastData.createViewTrainingDatasetActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);
        a = new ActionMap();

        actionsTrain.put("Similarity", a);
        chemCastData.createTrainingSimilarityActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);
        a = new ActionMap();
        actionsTrain.put("Descriptors", a);
        chemCastData.createTrainingDescriptorsActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);
        a = new ActionMap();
        actionsTrain.put("Groups", a);
        chemCastData.createTrainingSubsetActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);


        actionsTest = new Hashtable<String, ActionMap>();
        a = new ActionMap();
        actionsTest.put("File", a);
        chemCastData.createTestDatasetActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);

        a = new ActionMap();
        actionsTest.put("View", a);
        chemCastData.createViewTestDatasetActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);
        a = new ActionMap();

        a = new ActionMap();
        actionsTest.put("Similarity", a);
        chemCastData.createTestSimilarityActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);

        a = new ActionMap();
        actionsTest.put("Descriptors", a);
        chemCastData.createTestDescriptorsActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);
        a = new ActionMap();
        actionsTest.put("Groups", a);
        chemCastData.createTestSubsetActions(a, mainFrame);
        ActionFactory.setParentActions(a,allActions);

        /*
        visualActions = new Hashtable<String, ActionMap>();
        a = new ActionMap();
        visualActions.put("Visualization", a);
        ChemCASTActionFactory.createVisualActions(a, chemCastData,mainFrame);
        ActionFactory.setParentActions(a,allActions);
*/
	}

    protected JMenuBar createMenuBar() {
        // TODO Auto-generated method stub
        JMenuBar menubar = new JMenuBar();
        FileBatchProcessing batchAction = new FileBatchProcessing(chemCastData,mainFrame,"Process file");
        JMenu menu = new JMenu("File");
        menu.add(new JMenuItem(batchAction));
        menubar.add(menu);
        menu.add(new JMenuItem(new AbstractAction("Exit") {
        	public void actionPerformed(ActionEvent e) {
        		exit();

        	}
        }));
        menubar.add(menu);
        menubar.add(createMenu("Training set", actionsTrain));
        menubar.add(createMenu("Test set", actionsTest));
        menu = new JMenu("Help");
        AboutAction aboutAction = new AboutAction(getAboutString("chemCast.application"),mainFrame);
        aboutAction.setIconFile("chemCast/ui/images/jrc_logo.png");
        //aboutAction.setVersion("1.05");
        //TODO get version from package
       // aboutAction.setDevelopers("<br><font color='#0000FF'>Developed (2006,2007) by Ideaconsult Ltd.<br>for <b>European Chemicals Bureau</b></font><br>contact: Nina Jeliazkova nina@acad.bg</html>");
        menu.add(aboutAction);
        menu.add(new ConfigAction());
        menubar.add(menu);
        return menubar;
    }
    protected void exit() {
		 Runtime.getRuntime().runFinalization();
		 Runtime.getRuntime().exit(0);
    }
    protected JMenu createMenu(String caption, Hashtable<String,ActionMap> actions) {
          JMenu menu = new JMenu(caption);
          Enumeration e =  actions.keys();
          while (e.hasMoreElements()) {
              Object title = e.nextElement();
              JMenu aGroup = new JMenu(title.toString());
              ActionMap map = actions.get(title);


              Object[] keys = map.keys();
              for (int j=0;j<keys.length;j++)
                  aGroup.add(new JMenuItem(map.get(keys[j])));


              menu.add(aGroup);
          }

          return menu;
    }

	/* (non-Javadoc)
	 * @see ambit.ui.CoreApp#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent event) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see ambit.ui.CoreApp#createWidgets(javax.swing.JFrame, javax.swing.JPanel)
	 */
	protected void createWidgets(JFrame aFrame, JPanel aPanel) {
		aPanel.setLayout(new BorderLayout());

        MoleculesTable trainPanel = new MoleculesTable(
                new RandomAccessSubsetTableModel(null),new Dimension(150,150));
        String dataname = chemCastData.getDataset(ChemCASTModule.DATA_TRAINING).getName();
        chemCastData.getDataset(ChemCASTModule.DATA_TRAINING).addPropertyChangeListener(
                        dataname,
                        trainPanel);
        chemCastData.getDataset(ChemCASTModule.DATA_TRAINING).addPropertyChangeListener(
                IChemCastObject.p_properties,
                trainPanel);

        chemCastData.getDataset(ChemCASTModule.DATA_TRAINING).addPropertyChangeListener((StatusBar)statusBar);

        //chemCastData.addPropertyChangeListener(IChemCastObject.p_properties,trainPanel);
        chemCastData.addPropertyChangeListener(dataname,trainPanel);

        MoleculesTable testPanel = new MoleculesTable(
                new RandomAccessSubsetTableModel(null),new Dimension(150,150));
        dataname = chemCastData.getDataset(ChemCASTModule.DATA_TEST).getName();
        chemCastData.getDataset(ChemCASTModule.DATA_TEST).addPropertyChangeListener(
                dataname,
                testPanel);
        chemCastData.getDataset(ChemCASTModule.DATA_TEST).addPropertyChangeListener(
                IChemCastObject.p_properties,
                testPanel);
        //chemCastData.addPropertyChangeListener(IChemCastObject.p_properties,testPanel);
        chemCastData.addPropertyChangeListener(dataname,testPanel);
        chemCastData.getDataset(ChemCASTModule.DATA_TEST).addPropertyChangeListener((StatusBar)statusBar);

        JSplitPane moleculesPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                new TasksFrame(actionsTrain,"Training set",trainPanel),
                new TasksFrame(actionsTest,"Test set",testPanel));
        moleculesPane.setDividerLocation(400);
        //TasksFrame taskPanel = new TasksFrame(actionsTrain,"Tasks",moleculesPane);

        VisualizationPanel results_panel = new VisualizationPanel(chemCastData);

        JSplitPane mainPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,moleculesPane,results_panel);
        mainPanel.setDividerLocation(400);
		aPanel.add(mainPanel,BorderLayout.CENTER);

		toolBar.setVisible(true);
	}
/*
	 private void addSwitchViewButton(String title,Icon icon,
	   	      final Component component, JButtonBar bar, ButtonGroup group) {
	   	      Action action = new AbstractAction(title, icon) {
	   	        public void actionPerformed(ActionEvent e) {
	   	          //  System.out.println(((JToggleButton)e.getSource()).isSelected());
	   	          trainPanel = createMoleculesPanel(chemCastData.getMolecules(),((JToggleButton)e.getSource()).isSelected());
	   	          moleculesPane.setLeftComponent(trainPanel);
	   	        }
	   	      };

	   	    }




*/
	protected JComponent createToolbar() {
	    //JButtonBar toolbar = new JButtonBar(JButtonBar.HORIZONTAL);

        return new JToolBar();
	}
	protected JPanel createStatusBar() {
        StatusBar statusBar = new StatusBar();
		statusBar.setBorder(new BevelBorder(BevelBorder.LOWERED));
        chemCastData.addPropertyChangeListener(StatusBar.Message_Error,statusBar);
        chemCastData.addPropertyChangeListener(StatusBar.Message_Processing,statusBar);
        chemCastData.addPropertyChangeListener(StatusBar.Message_Progress,statusBar);
        chemCastData.addPropertyChangeListener(StatusBar.Message_Status,statusBar);
		//chemCastData.getJobStatus().addObserver((AmbitStatusBar)statusBar);
		//chemCastData.getBatchStatistics().addObserver((AmbitStatusBar)statusBar);
		return statusBar;

	}
    protected void doClose() {
        try {
            chemCastData.close();

        } catch (Exception x) {
            x.printStackTrace();
        }
        super.doClose();
   }
	/**
	 * @param args
	 */
	public static void main(final String[] args) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
                new ToxmatchApplication("Toxmatch",580,580,args);
            }
        });
	}
    protected ImageIcon getIcon() {
        try {
            ImageIcon icon = UITools.createImageIcon("ambit/ui/images/similarity_matrix.png");
            return icon;
        } catch (Exception x) {
            return null;
        }
    }
}



