/*
Copyright Ideaconsult Ltd. (C) 2005-2007  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
/**
 * 
 */
package chemCast.data;

import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.interfaces.IChemObject;

import ambit.similarity.ISimilarityProcessor;

/**
 * @author Nina Jeliazkova
 *
 */
public class PairwiseSimilarityFilter extends ProcessorFilter<ISimilarityProcessor> {
	protected IAtomContainer query;
	protected double threshold = 0.8;
	public static String[] conditions={">",">=","=","<=","<"};
	protected int conditionindex = 1;
	/**
	 * 
	 */
	public PairwiseSimilarityFilter() {
		super();
	}

	/**
	 * @param processor
	 */
	public PairwiseSimilarityFilter(ISimilarityProcessor processor) {
		super(processor);
	}
	@Override
	public void setProcessor(ISimilarityProcessor processor) {
		super.setProcessor(processor);
	}
	@Override
	public void setProperty(String key, Object value) {
		super.setProperty(key, value);
		if ("query".equals(key)) {
			if (value instanceof IAtomContainer)
				query = (IAtomContainer)value;
		} else if ("threshold".equals(key)) 
			try {
				threshold = Double.parseDouble(value.toString());
			} catch (Exception x) {
				x.printStackTrace();
				threshold = 0.8;
			}
	}
	protected boolean isProcessorValid(Object o) {
		return o instanceof ISimilarityProcessor;
	}	
	@Override
	public boolean accept(IChemObject object) {
		if (processor != null) {
			try {
				double d =   processor.predict(query, object);
				switch (conditionindex) {
				case 0: return d > threshold;
				case 1: return d >= threshold;
				case 2: return d== threshold;
				case 3: return d <= threshold;
				case 4: return d < threshold;
				default: return d >= threshold;				
				}
			} catch (Exception x) {
                x.printStackTrace();
				logger.error(x);
				return false;
			}
			
		} else return false;	
	}

	public IAtomContainer getQuery() {
		return query;
	}

	public void setQuery(IAtomContainer query) {
		this.query = query;
	}

	public double getThreshold() {
		return threshold;
	}

	public void setThreshold(double threshold) {
		this.threshold = threshold;
	}
	public String getCondition() {
		return conditions[conditionindex];
	}

	public void setCondition(String condition) {
		for (int i=0; i < conditions.length;i++)
			if (conditions[i].equals(condition)) {
				conditionindex = i;
				return;
			}
		//default-doesn't change
	}
	@Override
	public String toString() {
		return super.toString() + " " + conditions[conditionindex] + threshold;
	}	
}


