/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.data;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import org.openscience.cdk.interfaces.IChemObject;

import ambit.data.IAmbitEditor;
import chemCast.ui.ObjectPropertyFilterEditor;

public class RangePropertyFilter extends ObjectPropertyFilter  implements IPropertyFilter {
	protected double minValue;
	protected double maxValue;
	
	public static String p_minValue = "MinValue";
	public static String p_maxValue = "MaxValue";
	public RangePropertyFilter() throws Exception  {
		super();
	}
	public RangePropertyFilter(Object key, double minValue, double maxValue) throws Exception  {
		super(p_tag, key);
		setTag(key.toString());
		setMaxValue(maxValue);
		setMinValue(minValue);
	}
	@Override
	public boolean accept(IChemObject object) {
        Object key = parameters.get(p_tag);
        return acceptProperty(null, object.getProperty(key));
	}
	@Override
	public boolean acceptProperty(Object property, Object value) {
		double d;
		if (value instanceof Number) 
			d = ((Number) value).doubleValue();
		else 
		try {
			d = Double.parseDouble(value.toString());
		} catch (Exception x) {
			//x.printStackTrace();
			return false;
		}
		return (d >= minValue) &&  (d <= maxValue) ;
	}

	@Override
	public void setProperty(String key, Object value) {
		try {
			if (p_maxValue.equals(key)) maxValue = Double.parseDouble(value.toString());
			else if (p_minValue.equals(key)) minValue = Double.parseDouble(value.toString());
			else if (p_tag.equals(key)) {
				super.setProperty(p_tag,value);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	@Override
	public Object getProperty(String key) {
		if (key.equals(p_maxValue)) return maxValue;
		else if (key.equals(p_minValue)) return minValue;
		else return super.getProperty(key);
	}
	public Number getMaxValue() {
		return maxValue;
	}
	public void setMaxValue(double maxValue) {
		this.maxValue = maxValue;
	}
	public Number getMinValue() {
		return minValue;
	}
	public void setMinValue(double minValue) {
		this.minValue = minValue;
	}
	@Override
	public String toString() {
		return getTag() + " in (" + getMinValue() + "," + getMaxValue() + ")";
	}

	@Override
	public boolean equals(Object arg0) {
		RangePropertyFilter pf = (RangePropertyFilter) arg0;
		return getTag().equals(pf.getTag()) && 
			getMaxValue().equals(pf.getMaxValue()) &&
			getMinValue().equals(pf.getMinValue());
	}
	@Override
	public IAmbitEditor getEditor() {
		// TODO Auto-generated method stub
		return new RangePropertyFilterEditor("Range",this);
		}
}


class RangePropertyFilterEditor extends ObjectPropertyFilterEditor implements Comparator<Comparable> {
	protected JComboBox maxValueField;
	public RangePropertyFilterEditor(String title,  RangePropertyFilter filter) {
		this(new FlowLayout(),title,filter);
	}
	public RangePropertyFilterEditor(LayoutManager layout,String title,  RangePropertyFilter filter) {
		super(layout,title,filter);
		valueField.setSelectedItem(((RangePropertyFilter)filter).getMinValue().toString());
		maxValueField = new JComboBox(new String[] {((RangePropertyFilter)filter).getMaxValue().toString()});
        maxValueField.setPreferredSize(new Dimension(100,24));
        maxValueField.setMaximumSize(new Dimension(Integer.MAX_VALUE,24));
        maxValueField.setSelectedIndex(0);
		maxValueField.addActionListener(this);
		maxValueField.addFocusListener(this);
		//maxValueField.setPreferredSize(new Dimension(100,24));
		valueLabel.setText("Min");
		add(new JLabel("Max"));
		add(maxValueField);
	}
	@Override
	public void actionPerformed(ActionEvent e) {
        try {
                update(e.getSource());
            } catch (Exception x) {
                JOptionPane.showMessageDialog(this,x.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
            }

	}
	public void setEditable(boolean editable) {
		super.setEditable(editable);
		maxValueField.setEditable(editable);
	}
	@Override
	protected void update(Object source) throws Exception {
		if (source == keyField) {
			((ObjectPropertyFilter) object).setProperty(ObjectPropertyFilter.p_tag, keyField.getSelectedItem().toString());
            if (updateValues.isSelected() && (keyValuesUpdater!=null))
                populateValues(keyValuesUpdater.getValuesByField(keyField.getSelectedItem().toString()));
            
		} else 
		if (source == valueField) try {
			((RangePropertyFilter) object).setMinValue(Double.parseDouble(valueField.getSelectedItem().toString()));
		} catch (Exception x) {
            JOptionPane.showMessageDialog(this,"Not a number!","Error",JOptionPane.ERROR_MESSAGE);
            ((RangePropertyFilter) object).setMinValue(0);
        } else
			if (source == maxValueField) try {
				((RangePropertyFilter) object).setMaxValue(Double.parseDouble(maxValueField.getSelectedItem().toString()));
			}  catch (Exception x) {
                JOptionPane.showMessageDialog(this,"Not a number!","Error",JOptionPane.ERROR_MESSAGE);
                 ((RangePropertyFilter) object).setMaxValue(1);
            }
	}
    public void populateValues(Collection<Comparable> values) {
        if (values == null) return;
        try {
            Object o = Collections.max(values,this);
            
            maxValueField.setSelectedItem(o);
        } catch (Exception x) {
            maxValueField.setSelectedItem("1");
        }
        try {
            Object o = Collections.min(values,this);
            valueField.setSelectedItem(Collections.min(values,this));
        } catch (Exception x) {
            valueField.setSelectedItem("0");
        }        
        
        //TODO set min max
     //   valueField.setModel( new DefaultComboBoxModel(values.toArray()) ); 
    }
    public int compare(Comparable arg0, Comparable arg1) {
        try {
            return arg0.compareTo(arg1);
        } catch (Exception x) {
            return 0;
        }
    }

}

