/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.similarity.data;

import java.util.BitSet;

import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.qsar.model.QSARModelException;
import org.openscience.cdk.similarity.Tanimoto;

import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.structure.FingerprintGenerator;
import ambit.similarity.DoubleValuesHistogram;
import ambit.similarity.FingerprintSimilarity;
import ambit.stats.datastructures.Bin;
import chemCast.application.StatusBar;
import chemCast.weka.ISubsetsModel;

public class FingerprintSimilaritySubsets extends FingerprintSimilarity
		implements ISubsetsModel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -4229389238320443081L;
	protected Subsets datasets = null;
	
	public FingerprintSimilaritySubsets() {
		super();
		
	}

	public Subsets getDatasets() {
		return datasets;
	}

	public void setDatasets(Subsets datasets) {
		this.datasets = datasets;

	}

	public float predict(IAtomContainersList object1, int index1,
			IAtomContainersList object2, int index2) throws AmbitException {
		try {
			return Tanimoto.calculate(getBitset(object1, index1),getBitset(object2, index2));
		} catch (Exception x) {
			throw new AmbitException(x);
		}
	}

	public void build() throws QSARModelException {
		try {
			setTrainingPrefix(datasets.toString());
			buildInitialize();
	        for (int s=0; s < datasets.size();s++)
	            if (datasets.get(s).isEnabled()) 
	            try {
	                IAtomContainersList list = datasets.get(s).getData();
	                if (list == null) continue;
	                for (int i=0; i < list.getAtomContainerCount();i++) 
	                    try {
	                        datasets.firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/list.getAtomContainerCount()));
	            			BitSet bitset = getBitset(list, i);
	            			/*
	            			if (bitset != null)
	            				bitsets.add((BitSet)bitset);
	            				*/
	                    } catch (Exception x) {
	                        x.printStackTrace();
	                    }
	                    
	            } catch (Exception x) {
	                datasets.firePropertyChange(StatusBar.Message_Error,null,x);
	            }
			buildCompleted();
			setPredicting(true);
			setDatasets(null);
		} catch (Exception x) {
			setDatasets(null);
			throw new QSARModelException(x.getMessage());
		}
		

	}
	
	protected BitSet getBitset(IAtomContainersList list, int index) throws Exception {
		if (generator ==null) generator = new FingerprintGenerator();
		Object bitset = list.getProperty(index,AmbitCONSTANTS.Fingerprint);
		if ((bitset ==null) || ("".equals(bitset))) {
			IAtomContainer ac = list.getAtomContainer(index);
			
			generator.process(ac);
			bitset = ac.getProperty(AmbitCONSTANTS.Fingerprint);
			list.setProperty(index,AmbitCONSTANTS.Fingerprint,bitset);
		} else {
			generator.process(bitset);
		}
		if (bitset instanceof BitSet)
			return (BitSet)bitset;
		else return null;
	}

	public void predict() throws QSARModelException {
		if (!isPredicting()) return;
        //histogram.clear();
        if (modelStatistics != null) {
        	modelStatistics.clear(datasets.toString());
        	modelStatistics.setTitle(getSimilarityProperty());
        }
        datasets.firePropertyChange(StatusBar.Message_Status,null,"Prediction phase: " + toString() + " " + datasets.toString());
        String sp = getTrainingPrefix() + "." + getSimilarityProperty();
        for (int s=0; s < datasets.size();s++)
            if (datasets.get(s).isEnabled()) 
            try {
                IAtomContainersList list = datasets.get(s).getData();
                if (list == null) continue;
                for (int i=0; i < list.getAtomContainerCount();i++) 
                    try {
                        datasets.firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/list.getAtomContainerCount()));
        				BitSet bs = getBitset(list, i);
                        if (bs != null) {
                        	float tanimoto = Tanimoto.calculate(bs,consensusBitSet);
                            list.setProperty(i, sp,new Float(tanimoto));
                            //histogram.update(tanimoto);
                            
                            if (modelStatistics == null) modelStatistics = new DoubleValuesHistogram<Bin>(getSimilarityProperty());
                            modelStatistics.addValue(datasets.toString(),datasets.get(s).toString(), new Double(tanimoto));
                            
                        } else
                            list.setProperty(i, sp, null);
                        
                    } catch (Exception x) {
                        datasets.firePropertyChange(StatusBar.Message_Error,null,x);
                        list.setProperty(i, sp, null);
                    }
                    
            } catch (Exception x) {
                x.printStackTrace();
            }
         setDatasets(null);
	}

}


