/* SimilarityDefEditor.java
 * Author: Nina Jeliazkova
 * Date: Nov 26, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.ActionMap;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;

import ambit.data.IAmbitEditor;
import ambit.exceptions.AmbitException;
import ambit.ui.data.HashtableModel;
import chemCast.ui.ListPanel;

public class SimilarityDefEditor extends JPanel implements IAmbitEditor, ActionListener, FocusListener {
        /**
         * 
         */
        private static final long serialVersionUID = 8744303033654380960L;
        protected SimilarityDef object;
        protected JTextField nameField;
        protected ListPanel descriptorsPanel;
        protected boolean editable=false;
        protected JTabbedPane tabbedPane;
        
        public SimilarityDefEditor(SimilarityDef object) {
            super();
            tabbedPane = new JTabbedPane();
            setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
            nameField = new JTextField(object.getName());
            nameField.addActionListener(this);
            nameField.addFocusListener(this);
            nameField.setPreferredSize(new Dimension(100,24));
            nameField.setMaximumSize(new Dimension(Integer.MAX_VALUE,24));
            
            
            
            if (object.getDescriptors() !=null) {
                descriptorsPanel = new ListPanel("Available descriptors", object.getDescriptors());
                tabbedPane.addTab("Descriptors",descriptorsPanel);
            } 
            if (object.getParameters().size() > 0) {
                JTable table = new JTable(new HashtableModel(object.getParameters()) {
                    public String getColumnName(int arg0) {
                        switch (arg0) {
                        case 0: return "Parameter";
                        case 1: return "Value";
                        }
                        return "";
                    }
                    @Override
                    public boolean isCellEditable(int rowIndex, int columnIndex) {
                        return columnIndex==1;
                    }
                });
                JScrollPane sp = new JScrollPane(table);
                sp.setPreferredSize(new Dimension(200,150));
                sp.setMinimumSize(new Dimension(100,100));
                tabbedPane.addTab("Parameters",sp);
            }
            if (tabbedPane.getTabCount() > 0) {
            	add(nameField,BorderLayout.NORTH);
            	add(tabbedPane,BorderLayout.CENTER);
            } else
            	add(nameField,BorderLayout.CENTER);
        }
        public JComponent getJComponent() {
            return this;
        }
        public boolean view(Component parent, boolean editable,String title) throws AmbitException {
            setEditable(editable);
            return JOptionPane.showConfirmDialog(parent, getJComponent(),title,JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE) == JOptionPane.OK_OPTION;
        }
        public boolean isEditable() {
            return editable;
        }
        public void setEditable(boolean editable) {
            this.editable = editable;
            nameField.setEditable(editable);
        }
        protected void update(Object source) {
            if (source == nameField) {
                object.setName(nameField.getText());
            }
        }
        public void actionPerformed(ActionEvent e) {
            update(e.getSource());
            
        }
        
        public void focusLost(FocusEvent e) {
            update(e.getSource());
        }
        public void focusGained(FocusEvent e) {
            // TODO Auto-generated method stub
            
        }
        public void setDescriptorActions(ActionMap actions) {
            if (descriptorsPanel != null)
            descriptorsPanel.setActions(actions);
        }
 }
