/* SimilarityModel.java
 * Author: Nina Jeliazkova
 * Date: Nov 17, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.w3c.dom.Element;

import ambit.data.IAmbitEditor;
import ambit.processors.IAmbitResult;
import ambit.processors.results.AmbitResultsList;
import ambit.similarity.ISimilarityProcessor;

public class SimilarityModel extends SimilarityDef implements PropertyChangeListener {
    protected int[] subsetIndices = null;
    protected ISimilarityProcessor processor = null;
    protected AmbitResultsList results = null;
    public SimilarityModel() {
        super();
    }

    public SimilarityModel(String name, String type, String groups,
            String classname) {
        super(name, type, groups, classname);

    }

    public SimilarityModel(Element element) throws Exception {
        super(element);
    }
    public SimilarityModel(SimilarityDef similarity) {
        super(similarity.getName(),similarity.getType(),similarity.getGroups(),similarity.getClassName());
        setDescriptors(similarity.getDescriptors());
        setParameters(similarity.getParameters());
        
    }
    @Override
    public ISimilarityProcessor createSimilarityProcessor() throws Exception {
    	if (processor == null) {
    		processor = super.createSimilarityProcessor();
    		//processor.addPropertyChangeListener(this);
            firePropertyChange(p_similarity, null, this);
    	}
    	processor.setSimilarityProperty(getName());
    	return processor;
    }
    
    public synchronized int[] getSubsetIndices() {
        return subsetIndices;
    }

    public synchronized void setSubsetIndices(int[] subsetIndices) {
        this.subsetIndices = subsetIndices;
        firePropertyChange(p_similarity, null, this);
    }
    /**
     * have to be sorted in the same direction
     * @param index1 int[]
     * @param index2 int[]
     * @return
     */
    protected boolean equals(int[] index1, int[] index2) {
        if (index1.length!=index2.length) return false;
        for (int i=0; i < index1.length;i++)
            if (index1[i] != index2[i]) return false;
        return true;
    }
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof SimilarityModel) {
            int[] indices = ((SimilarityModel)obj).getSubsetIndices();
            if ((subsetIndices == null) && (indices==null)) return super.equals(obj);
            if ((subsetIndices != null) && (indices!=null)) {
                return equals(subsetIndices,indices) && super.equals(obj);
            } else return false;
        } else return super.equals(obj);
        
    }

    @Override
    public int hashCode() {
    	int hash = 7;
    	int var_code = super.hashCode();
    	hash = 31 * hash + var_code; 
    	if (subsetIndices ==null) {
    		var_code=0;
    		hash = 31 * hash + var_code;
    	}
    	else {
    		var_code = subsetIndices.length;
    		hash = 31 * hash + var_code;
    		for (int i=0; i < subsetIndices.length;i++) {
        		hash = 31 * hash + subsetIndices[i];    			
    		}
    	}
    	return hash;
    	
    }
    public void propertyChange(PropertyChangeEvent evt) {
    	if (evt.getNewValue() instanceof IAmbitResult) {
    		if (results == null) results = new AmbitResultsList();
    		results.add((IAmbitResult) evt.getNewValue());
    	}
    	
    }
    @Override
    public void clear() {
    	if (processor !=null) processor.close(); processor = null;
        if (results != null)
        results.clear();
        setSubsetIndices(null);
    	super.clear();
        firePropertyChange(p_similarity, null, this);
    }

	public ISimilarityProcessor getProcessor() {
		return processor;
	}
	@Override
	public IAmbitEditor getEditor() {
	    return new SimilarityModelEditor(this);
	}
    @Override
    public String toString() {
    	if (getSubsetIndices().length==1)
    		return super.toString() + " [prediction]";
    	else
    		return super.toString() + " [classification]";
    }
}

class SimilarityModelEditor extends SimilarityDefEditor {
    public SimilarityModelEditor(SimilarityModel model) {
        super(model);
        if (model.getProcessor() != null)
            add(model.getProcessor().getEditor().getJComponent());
    }
}