/* Subset.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.TableCellRenderer;

import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.io.IChemObjectWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ambit.data.IAmbitEditor;
import ambit.data.molecule.IAtomContainersList;
import ambit.io.FileWithHeaderWriter;
import ambit.io.MDLWriter;
import ambit.ui.data.HashtableModel;
import chemCast.application.StatusBar;
import chemCast.data.FilteredMoleculesFile;
import chemCast.data.IObjectFilter;
import chemCast.exception.FilterException;
import chemCast.exception.SubsetException;

public class Subset extends Dataset {
    public final static String p_group = "group";
    protected String name= null;
    protected Hashtable parameters;
    protected String filterClass = "chemCast.data.PropertyFilter";
    
    protected transient IAtomContainersList parentDataset = null;
    
    public Subset() throws SubsetException {
        this("Default",null,Color.red);
    }
    
    public Subset(Element element) throws Exception {
        this();
        setXMLElement(element);
        setParentDataset(null,null);
    }
    
    public Subset(String name, Hashtable parameters,Color color) throws SubsetException {
        super();
        setName(name);
        this.parameters = new Hashtable();
        if (parameters != null) this.parameters.putAll(parameters);
        setColor(color);
        setParentDataset(null,null);
    }
    @Override
    protected void finalize() throws Throwable {
    	super.finalize();
    }

    public synchronized String getName() {
        return name;
    }
    public synchronized void setName(String name) {
        firePropertyChange(p_name, this.name,name);
        this.name = name;

    }
    @Override
    public String toString() {
        return name;
    }
    @Override
    /**
     * Should generate XML element like this
     * <pre>
     * <group name="Inert chemicals (Narcotics)" tag="MOA" label="Narcotics" color="#FF0000"/>
     * </pre> 
     */
    public Element getXMLElement(Document document) {
        Element e = document.createElement(p_group);
        e.setAttribute(p_name, getName());
        Enumeration i = parameters.keys();
        while (i.hasMoreElements()) {
        	Object key = i.nextElement();
            Object values = parameters.get(key);
            if (values instanceof Collection) {
                Element parameters = document.createElement(p_parameters);
                parameters.setAttribute(p_name, key.toString());
                e.appendChild(parameters);
                Iterator it = ((Collection) values).iterator();
                while (it.hasNext()) {
                    Element parameter = document.createElement(p_parameter);
                    parameter.setAttribute(key.toString(), it.next().toString());
                    parameters.appendChild(parameter);
                }
            } else
                e.setAttribute(key.toString(),parameters.get(key).toString());
        }
        e.setAttribute(p_class,filterClass);
        e.setAttribute(p_color, Color2Hex(getColor()));
        return e;
    }
    /**
     * Expects element like
     * <pre>
     * <group name="Inert chemicals (Narcotics)" tag="MOA" label="Narcotics" color="#FF0000"/>
     * </pre>  
     * Updates name, label and color of the {@link Subset}.
     */
    public void setXMLElement(Element element) throws Exception {
        if (element.getNodeName().equals(p_group)) {
            setName(element.getAttribute(IChemCastObject.p_name));
            setFilterClass(element.getAttribute(IChemCastObject.p_class));
            parameters.clear();
            NamedNodeMap attributes = element.getAttributes();
            for (int i=0; i <attributes.getLength(); i++) {
            	Node key = attributes.item(i);
            	setProperty(key.getNodeName(),key.getNodeValue());
            }
            String color = element.getAttribute(Subset.p_color);
            if (color != null)
                try {
                    setColor(Hex2Color(color));
                } catch (Exception x) {
                    //x.printStackTrace();
                    setColor(Color.gray);
                }
            else setColor(Color.blue);
            
            NodeList child = element.getElementsByTagName(p_parameters);
            for (int i=0; i<child.getLength();i++) {
                Node node = child.item(i);
                if (node instanceof Element) {
                    String key = ((Element)node).getAttribute(Subset.p_name);
                    NodeList param = ((Element)node).getElementsByTagName(p_parameter);
                    HashSet<String> values = new HashSet<String>();
                    for (int j=0; j<param.getLength();j++) {
                        String value = ((Element)param.item(j)).getAttribute(key);
                        if (value != null) values.add(value);
                    }
                    if (values.size()>0)
                        setProperty(key,values);
                }
            }
            
            
        } else throw new Exception("Not an element of type "+p_group);
    }
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Subset) {
            Subset s = (Subset) obj;
            return s.getName().equals(getName()) &&
                    s.getColor().equals(getColor());
        } else return false;
    }
    public int compareTo(Object o) {
        // TODO Auto-generated method stub
        return 0;
    }
	public IAtomContainersList getParentDataset() {
		return parentDataset;
	}
	public void setParentDataset(IAtomContainersList list,IObjectFilter filter) throws SubsetException {
		this.parentDataset = list;
		setData(null);
		if (list != null)
            try {
                IAtomContainersList data = createData(parentDataset,filter); 
    			setData(data);
            } catch (FilterException x) {
                throw new SubsetException(x);
            }
		firePropertyChange(p_group, null, this);
	}

	
	public IObjectFilter createFilter() throws FilterException {
		IObjectFilter o = null;
		try {
			Class c = Class.forName(filterClass);
			o = (IObjectFilter)c.newInstance();
			((IObjectFilter) o).setProperties(parameters);
			return o;
		} catch (Exception x) {
			throw new FilterException(x);
		}		
	}
	public IAtomContainersList createData(IAtomContainersList parentdata, IObjectFilter filter) throws FilterException  {
		IObjectFilter o;
		if (filter == null)
			o = createFilter();
		else {
			o = filter;
			setFilterClass(o.getClass().getName());
		}
		if (o != null) {
			return new FilteredMoleculesFile(parentdata,(IObjectFilter) o);
		} else throw new FilterException("Can't create subset filter");
	}
	@Override
	public IAtomContainersList getData() throws SubsetException {
		//if ((parentDataset == null) ) throw new SubsetException("Parent dataset not defined!");
		return data;
	}
	@Override
	public void setData(IAtomContainersList data) throws SubsetException {
		if ((this.data != null) && (this.data instanceof FilteredMoleculesFile))
			((FilteredMoleculesFile) this.data).close();
			
		this.data = data;
	}
	public Object getProperty(String key) {
		return parameters.get(key);
	}
	public void setProperty(String key, Object value) {
		if (value != null) {
			firePropertyChange(key, getProperty(key), value);
	        parameters.put(key, value);
		}
	}
    @Override
    protected Object clone() throws CloneNotSupportedException {
    	try {
	        Subset s = new Subset(name,parameters,color);
	        s.setFilterClass(filterClass);
	        //This is a pointer, not a copy!
	        s.setParentDataset(getParentDataset(),null);
	        return s;
    	} catch (SubsetException x) {
    		throw new CloneNotSupportedException(x.toString());
    	}
    }

	public String getFilterClass() {
		return filterClass;
	}

	public void setFilterClass(String filterClass) {
		firePropertyChange(p_class, this.filterClass, filterClass);
		this.filterClass = filterClass;
	}

    public synchronized Hashtable getParameters() {
        return parameters;
    }
    @Override
    public IAmbitEditor getEditor() {
        return new SubsetEditor("Group details",this);
    }
    public void write(IChemObjectWriter writer, ArrayList fields) {
        try {
            Hashtable p = new Hashtable(); 
            IAtomContainersList data = getData();
            if (data == null) return;
            firePropertyChange(StatusBar.Message_Status,null,"Saving " + data.toString() + " to " + writer.toString());
            for (int i=0; i < data.getAtomContainerCount(); i++)
                 try {
                     firePropertyChange(StatusBar.Message_Progress,null,new Integer(100*i/data.getAtomContainerCount()));
                     IAtomContainer a = data.getAtomContainer(i);
                     p.clear();
                     for (int f=0; f < fields.size();f++) {
                         Object value = data.getProperty(i,fields.get(f));
                         if (value != null)
                             p.put(fields.get(f), value);
                     }    

                     if (writer instanceof MDLWriter) {
                         ((MDLWriter) writer).setSdFields(p);
                     } else if (writer instanceof FileWithHeaderWriter) {
                         ((FileWithHeaderWriter)writer).setHeader(fields);
                     }
                     writer.write(a);
                 } catch (Exception x) {
                     firePropertyChange(StatusBar.Message_Error,null,x);
                 }
        } catch (Exception x) {
            firePropertyChange(StatusBar.Message_Error,null,x);            
        }
    }
	
}

class SubsetEditor extends DatasetEditor {
    /**
	 * 
	 */
	private static final long serialVersionUID = -2334416648559496053L;
	protected boolean editable;
    
    public SubsetEditor(String title,Subset object) {
        super(title,object);
        editable = false;
        JComponent params;
        if (object.getParameters().size() > 0) {
            JTable table = new JTable(new HashtableModel(object.getParameters()) {
                public String getColumnName(int arg0) {
                    switch (arg0) {
                    case 0: return "Parameter";
                    case 1: return "Value";
                    }
                    return "";
                }
                @Override
                public boolean isCellEditable(int rowIndex, int columnIndex) {
                    return (columnIndex==1) && isEditable();
                }
            }) {
            	@Override
            	public TableCellRenderer getCellRenderer(int row, int column) {
            		TableCellRenderer r =  super.getCellRenderer(row, column);
            		setToolTipText(getValueAt(row, column).toString());
            		return r;
            	}
            };
            
            params = new JScrollPane(table);
            params.setPreferredSize(new Dimension(200,200));
            params.setMinimumSize(new Dimension(200,100));
            
        } else { params = new JTextField("NONE");
            ((JTextField)params).setEditable(false);
            ((JTextField)params).setText("NONE");
            params.setPreferredSize(new Dimension(200,24));
            params.setMinimumSize(new Dimension(200,24));
        }
        params.setToolTipText("Parameters");
        add(new JLabel("Parameters"));
        
        add(params);
    }
    public boolean isEditable() {
    	return editable;
    }
    @Override
    public void setEditable(boolean editable) {
        super.setEditable(editable);
        this.editable = editable;
    }
    @Override
    public void actionPerformed(ActionEvent e) {
    	super.actionPerformed(e);
    	if (e.getActionCommand().equals("name")) {
    		((Subset)object).setName(nameField.getText());
    	}
    }
    @Override
    public void focusLost(FocusEvent e) {
    	if (e.getSource() ==nameField) {
    		((Subset)object).setName(nameField.getText());
    	}
    }
}    
