package chemCast.test.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;

/**
 * A demo showing the addition and removal of multiple datasets / renderers.
 */
public class MultipleDatasets extends ApplicationFrame implements ActionListener {

    /**
     * 
     */
    private static final long serialVersionUID = -730219015089941730L;

    /** The plot. */
    private XYPlot plot;
   
    /** The index of the last dataset added. */
    private int datasetIndex = 0;
    
    /**
     * Constructs a new demonstration application.
     *
     * @param title  the frame title.
     */
    public MultipleDatasets(final String title) {

        super(title);
        final XYSeriesCollection dataset1 = createRandomDataset("Series 1");
        
        XYPlot plot = new XYPlot(dataset1,new NumberAxis("X"),new NumberAxis("Y"),
                new StandardXYItemRenderer(StandardXYItemRenderer.SHAPES));
        final JFreeChart chart = new JFreeChart("Demo",plot); 
        chart.setBackgroundPaint(Color.white);
        
        this.plot = chart.getXYPlot();
        this.plot.setBackgroundPaint(Color.lightGray);
        this.plot.setDomainGridlinePaint(Color.white);
        this.plot.setRangeGridlinePaint(Color.white);
//        this.plot.setAxisOffset(new Spacer(Spacer.ABSOLUTE, 4, 4, 4, 4));
        final ValueAxis axis = this.plot.getDomainAxis();
        axis.setAutoRange(true);

        final NumberAxis rangeAxis2 = new NumberAxis("Range Axis 2");
        rangeAxis2.setAutoRangeIncludesZero(false);
        rangeAxis2.setAutoRange(true);
        
        final JPanel content = new JPanel(new BorderLayout());

        final ChartPanel chartPanel = new ChartPanel(chart);
        content.add(chartPanel);
        
        final JButton button1 = new JButton("Add Dataset");
        button1.setActionCommand("ADD_DATASET");
        button1.addActionListener(this);
        
        final JButton button2 = new JButton("Remove Dataset");
        button2.setActionCommand("REMOVE_DATASET");
        button2.addActionListener(this);

        final JPanel buttonPanel = new JPanel(new FlowLayout());
        buttonPanel.add(button1);
        buttonPanel.add(button2);
        
        content.add(buttonPanel, BorderLayout.SOUTH);
        chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        setContentPane(content);

    }

    /**
     * Creates a random dataset.
     * 
     * @param name  the series name.
     * 
     * @return The dataset.
     */
    private XYSeriesCollection createRandomDataset(final String name) {
        final XYSeries series = new XYSeries(name);
        double value = 100.0;
        
        int n = (int) (50*Math.random());
        for (int i = 0; i < n; i++) {
            series.add(i + 10*Math.random(), value);
            
            value = value * (1.0 + Math.random() / 100);
        }
        final XYSeries series1 = new XYSeries(name+".1");
        
        
        for (int i = 0; i < n; i++) {
            series1.add(i + 10*Math.random(), value);
            
            value = value * (10 + Math.random() / 100);
        }
        XYSeriesCollection c = new XYSeriesCollection();
        c.addSeries(series);
        c.addSeries(series1);
        return c;
    }
    
    // ****************************************************************************
    // * JFREECHART DEVELOPER GUIDE                                               *
    // * The JFreeChart Developer Guide, written by David Gilbert, is available   *
    // * to purchase from Object Refinery Limited:                                *
    // *                                                                          *
    // * http://www.object-refinery.com/jfreechart/guide.html                     *
    // *                                                                          *
    // * Sales are used to provide funding for the JFreeChart project - please    * 
    // * support us so that we can continue developing free software.             *
    // ****************************************************************************
    
    /**
     * Handles a click on the button by adding new (random) data.
     *
     * @param e  the action event.
     */
    public void actionPerformed(final ActionEvent e) {
       
        if (e.getActionCommand().equals("ADD_DATASET")) {
            if (this.datasetIndex < 20) {
                this.datasetIndex++;
                this.plot.setDataset(
                    this.datasetIndex, createRandomDataset("S" + this.datasetIndex)
                );
                this.plot.setRenderer(this.datasetIndex, new StandardXYItemRenderer(StandardXYItemRenderer.SHAPES));
            }
        }
        else if (e.getActionCommand().equals("REMOVE_DATASET")) {
            if (this.datasetIndex > 1) {
                this.plot.setDataset(2, null);
                this.plot.setRenderer(2, null);
                this.datasetIndex--;
            }
        }
        
    }

    /**
     * Starting point for the demonstration application.
     *
     * @param args  ignored.
     */
    public static void main(final String[] args) {

        final MultipleDatasets demo = new MultipleDatasets("Multiple Dataset Demo 1");
        demo.pack();
        RefineryUtilities.centerFrameOnScreen(demo);
        demo.setVisible(true);

    }

}
