/* MyChart.java
 * Author: Nina Jeliazkova
 * Date: Nov 12, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JToolBar;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.labels.StandardXYItemLabelGenerator;
import org.jfree.chart.labels.StandardXYSeriesLabelGenerator;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.Range;
import org.jfree.data.xy.XYDataset;
import org.jfree.util.ShapeUtilities;

import ambit.data.molecule.PropertyTranslator;
import chemCast.data.MultiplePropertyFilters;
import chemCast.data.RangePropertyFilter;
import chemCast.data.SelectedItem;
import chemCast.events.SelectItemEvent;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;

public class MyChart extends JPanel implements ActionListener,ChartMouseListener {
	protected final ChartPanel chartPanel;
    protected SubsetsXYPlot plot;
    protected Vector[] properties;
    protected JComboBox[] box;
    protected SelectedItem selectedItem;
    protected String colorProperty = "groups";
    protected String labelProperty = "";
    protected Hashtable<String, Subset> filteredSubsets; 


    /**
     * 
     */
    private static final long serialVersionUID = 1653059686469797315L;
    public MyChart(Subsets[] subsets,Object propertyX, Object propertyY, Object propertyZ, 
            Object labelProperty, Object tooltipProperty, Object colorProperty) {
        super(new BorderLayout());
        filteredSubsets = new Hashtable<String, Subset>();
        
        chartPanel = createChart(subsets, propertyX, propertyY, propertyZ, labelProperty, tooltipProperty,colorProperty);
        chartPanel.setFillZoomRectangle(false);
        chartPanel.addChartMouseListener(this);
        add(chartPanel,BorderLayout.CENTER);
        add(createControl(new String[] {" X "," Y "}),BorderLayout.SOUTH);
        
    }
    protected ChartPanel createChart(Subsets[] subsets,
            Object propertyX, Object propertyY, Object propertyZ, Object labelProperty, Object tooltipProperty,Object colorProperty) {
        
        MoleculesXYZDataset[] dataset = new MoleculesXYZDataset[subsets.length];
        for (int i=0; i < subsets.length;i++) {
            dataset[i] = new MoleculesXYZDataset(subsets[i],
                  propertyX, propertyY, propertyZ, labelProperty, tooltipProperty,colorProperty);
        }
        plot = new SubsetsXYPlot(dataset,
                new XYItemRenderer[] {createRenderer(subsets[0],ShapeUtilities.createDownTriangle(2),new Rectangle(-2,-2,4,4))
                ,
                createRenderer(subsets[1],ShapeUtilities.createUpTriangle(2),ShapeUtilities.createDiamond(2))
                });
        
        for (int i=0; i < subsets.length;i++) {
            subsets[i].addPropertyChangeListener(plot);
        }
        JFreeChart chart = new JFreeChart(plot);
        //chart.removeLegend();
        
        chart.setAntiAlias(false);
        return new ChartPanel(chart);
    }    
    protected XYItemRenderer createRenderer(Subsets subset,Shape subsetShape,Shape fileShape) {    
        XYItemRenderer renderer = new MoleculesXYZRenderer(subset,subsetShape,fileShape
                );

        renderer.setToolTipGenerator(new StandardXYToolTipGenerator() {
            /**
             * 
             */
            private static final long serialVersionUID = -8934128088291422557L;

            public String generateToolTip(XYDataset dataset, int series,
                    int item) {
                return ((MoleculesXYZDataset) dataset).getTooltip(series, item);
            };
        });
        renderer.setItemLabelGenerator(new StandardXYItemLabelGenerator() {
            /**
             * 
             */
            private static final long serialVersionUID = 788677374614236001L;

            @Override
            public String generateLabelString(XYDataset dataset, int series,
                    int item) {
                return ((MoleculesXYZDataset) dataset).getLabel(series,item);
                //return "";// ((MoleculesXYZDataset)
                            // dataset).getZ(series,item).toString();
            }
            
            @Override
            public String generateLabel(XYDataset dataset, int series, int item) {
                return ((MoleculesXYZDataset) dataset).getLabel(series,item);
            }

        });
        
        renderer.setLegendItemLabelGenerator(new StandardXYSeriesLabelGenerator() {
            /**
             * 
             */
            private static final long serialVersionUID = -5183943706370411631L;

            @Override
            public String generateLabel(XYDataset arg0, int arg1) {
                try {
                    return arg0.getSeriesKey(arg1).toString();
                } catch (Exception x) {
                    x.printStackTrace();
                    return arg0 + " #" + arg1 + "/" + arg0.getSeriesCount();
                }   
            }
        });
            
        
        renderer.setItemLabelsVisible(true);
        return renderer;
    }   
    protected JComponent createControl(String[] titles) {
        Dimension labelD = new Dimension(64,24);
        Dimension optionsD = new Dimension(80,24);
        Dimension boxD = new Dimension(Integer.MAX_VALUE,24);
        JToolBar tb = new JToolBar();

        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel,BoxLayout.PAGE_AXIS));
        

        //p.setLayout(new BoxLayout(p,BoxLayout.LINE_AXIS));
        double w = 2;
        int num = titles.length;
        properties = new Vector[num];
        box = new JComboBox[num];
        for (int i=0; i < num; i++) {
            JPanel p = new JPanel(new BorderLayout());
            //p.setMaximumSize(new Dimension(Integer.MAX_VALUE,12));
            properties[i] = new Vector();
            box[i] = new JComboBox(properties[i]);
            box[i].setRenderer(new BasicComboBoxRenderer() {
            	/**
                 * 
                 */
                private static final long serialVersionUID = -7935685349051302844L;

                @Override
            	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            		Component c =  super.getListCellRendererComponent(list, value, index, isSelected,
            				cellHasFocus);
            		if (value != null) 
            			list.setToolTipText(value.toString());
            		return c;
            	}
            });
            box[i].setActionCommand(Integer.toString(i));
            box[i].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JComboBox cb = (JComboBox)e.getSource();
                    String c = (String)cb.getSelectedItem();
                    int index = Integer.parseInt(cb.getActionCommand());
                    
                    ((SubsetsXYPlot)plot).setProperty(index,c);
                    switch (index) {
                    case 0: {plot.getDomainAxis().setLabel(c);break;}
                    case 1: {plot.getRangeAxis().setLabel(c);break;}
                    //case 2: {((SubsetsXYPlot)plot).setProperty(2, property).;break;}
                    }
                      }
            });
            box[i].setMaximumSize(boxD);
            box[i].setPreferredSize(boxD);
            //box[i].setMinimumSize(new Dimension(50,12));
            JLabel labelX = new JLabel(titles[i]);
            if (i < 2)
            	labelX.setToolTipText("Controls what will be displayed on " + titles[i] + " axis. Click on radioboxes at the bottom, to fill in X and Y lists with the available options.");
            	
            //labelX.setMinimumSize();
            //labelX.setPreferredSize(new Dimension(32,12));
            labelX.setMaximumSize(labelD);
            labelX.setPreferredSize(labelD);
            labelX.setLabelFor(box[i]);
            
            

            p.add(labelX,BorderLayout.WEST);
            
            switch (i) {
            case 1: {    JCheckBox logBox = new JCheckBox("(Log)");
	            logBox.setActionCommand(Integer.toString(i));
	            logBox.setToolTipText("If checked, Y axis will switch to a logarithmic one.");
	            logBox.setSelected(false);
	            logBox.addItemListener(new ItemListener() {
	            	public void itemStateChanged(ItemEvent e) {
	            		plot.setLogAxis(((AbstractButton)e.getSource()).isSelected());
	            	}
	            });
	            logBox.setPreferredSize(optionsD);
	            logBox.setMaximumSize(optionsD);
	            p.add(logBox,BorderLayout.EAST);
	            break;
            }
            }
            
            p.add(box[i],BorderLayout.CENTER);
            panel.add(p);

        }
        
        JPanel p = new JPanel();       //p.setFloatable(true);     
            ButtonGroup g = new ButtonGroup();
            JRadioButton b = new JRadioButton("Training/Test similarity");
            b.setToolTipText("Click here to fill in X and Y lists with available similarity measures.");
            b.setActionCommand("Compare");
            g.add(b);
            p.add(b);
            b.addActionListener(this);
            b = new JRadioButton("Descriptors");
            b.setToolTipText("Click here to fill in X and Y lists with available descriptor names.");
            b.setActionCommand("Descriptors");
            b.addActionListener(this);
            g.add(b);
            p.add(b);
            
            b = new JRadioButton("Similarity vs. Activity");
            b.setToolTipText("Click here to fill in X list with available similarity measures and Y list with available activity name.");
            b.setActionCommand("Activity");
            b.addActionListener(this);
            g.add(b);
            p.add(b);
            
            b = new JRadioButton("Descriptors vs. Activity");
            b.setToolTipText("Click here to fill in X list with available descriptor names and Y list with available activity name.");
            b.setActionCommand("qsar");
            b.addActionListener(this);
            g.add(b);
            p.add(b);            
            panel.add(p);
            
            p = new JPanel(); 
            JButton cb = new JButton("Color");
            cb.setToolTipText("Controls how points will be colored.");
            cb.setPreferredSize(optionsD);
            cb.setMaximumSize(optionsD);
            cb.setActionCommand("Color");
            cb.addActionListener(this);
            p.add(cb);

            cb = new JButton("Labels");
            cb.setToolTipText("Controls how points will be labeled.");
            cb.setPreferredSize(optionsD);
            cb.setMaximumSize(optionsD);
            cb.setActionCommand("Labels");
            cb.addActionListener(this);
            p.add(cb);
            
            cb = new JButton("Show ");
            cb.setToolTipText("Display selected chemicals.");
            cb.setPreferredSize(optionsD);
            cb.setMaximumSize(optionsD);
            cb.setActionCommand("Show");
            cb.addActionListener(this);
            p.add(cb);
            
            
        panel.add(p);            
        tb.add(panel);
        return tb;
    }

    public void actionPerformed(ActionEvent e) {
        if (plot.getDataset(1) == null) return;
        if (!(plot.getDataset(1) instanceof MoleculesXYZDataset)) return; 
        MoleculesXYZDataset xyzdataset = ((MoleculesXYZDataset)plot.getDataset(1));

        Subsets subsets = xyzdataset.getReaders();
        if (subsets == null) return;
        DatasetDef data = subsets.getParentDataset();
        if (data == null) return;
        if (e.getActionCommand().equals("Compare")) {
            if (data.getProperties().getProperties("similarity")!=null) {
                Collection snames = sort(data.getProperties().getProperties("similarity").values());                
                setProperty(box[0],snames);
                setProperty(box[1],snames);
            }
        } else
        if (e.getActionCommand().equals("Descriptors")) {
            Collection dnames = sort(data.getProperties().getDescriptorProperties().values());
            setProperty(box[0],dnames);
            setProperty(box[1],dnames);
        } else
        if (e.getActionCommand().equals("Activity")) {
            if (data.getProperties().getProperties("endpoint")!=null)
            setProperty(box[1],data.getProperties().getProperties("endpoint").values());
            if (data.getProperties().getProperties("similarity")!=null) {
                Collection snames = sort(data.getProperties().getProperties("similarity").values());
                setProperty(box[0],snames);
            }
        } else
        if (e.getActionCommand().equals("qsar")) {
                if (data.getProperties().getProperties("endpoint")!=null)
                setProperty(box[1],data.getProperties().getProperties("endpoint").values());
                Collection dnames = sort(data.getProperties().getDescriptorProperties().values());
                setProperty(box[0],dnames);
        } else        	
        if (e.getActionCommand().equals("Color")) {
           Object[] options = data.getProperties().enumerateProperties( new String[] {"similarity"}, "groups");
           Object result = JOptionPane.showInputDialog(this, "Color by",  "Select how the points should be colored", 
                   JOptionPane.PLAIN_MESSAGE, null, options, colorProperty);
           if (result != null) {
               colorProperty = result.toString();
               ((SubsetsXYPlot)plot).setProperty(2,colorProperty);
           }    
        } else  if (e.getActionCommand().equals("Labels")) {
            Object[] options = data.getProperties().enumerateProperties( 
                    new String[] {PropertyTranslator.type_identifiers,"similarity",PropertyTranslator.type_results,"subsets",PropertyTranslator.type_descriptors},"NONE");
            Object result = JOptionPane.showInputDialog(this, "Label by",  "Select how the points should be labeled", 
                    JOptionPane.PLAIN_MESSAGE, null, options, labelProperty);
            if (result != null) {
                labelProperty = result.toString();
                ((SubsetsXYPlot)plot).setProperty(3,labelProperty);
            }    
        }  else  if (e.getActionCommand().equals("Show")) {

        	try {
        		MultiplePropertyFilters filter = new MultiplePropertyFilters();
        		Range r = plot.getDomainAxis().getRange();
        		filter.addFilter(
        				new RangePropertyFilter(plot.getProperty(0).toString(),r.getLowerBound(),r.getUpperBound()));
        		r = plot.getRangeAxis().getRange();
        		filter.addFilter(
        				new RangePropertyFilter(plot.getProperty(1).toString(),r.getLowerBound(),r.getUpperBound()));
    	        
    	        
    	        for (int i=0; i < plot.getDatasetCount(); i++) 
    	        	if (plot.getDataset(i) instanceof MoleculesXYZDataset) 
    	        	try {
    	        		
    	        		String key = ((MoleculesXYZDataset) plot.getDataset(i)).getReaders().getType();
    	        		
    	        		Subset subset = filteredSubsets.get(key);
    	        		if (subset != null) {
    	        			subset.clear();
    	        		}
    	        		subset = new Subset(filter.toString(),null,Color.green);
    	    	        subset.setFilterClass("chemCast.data.MultiplePropertyFilters");
    	        		filteredSubsets.put(key, subset);
    	        		
		    	        Subsets d = ((MoleculesXYZDataset) plot.getDataset(i)).getReaders();
		    	        d.createNewSubset(subset,filter);
		    	        d.firePropertyChange(new SelectItemEvent(
		                        this,
		                        "",
		                        null,
		                        new SelectedItem(d,subset,0)
		                        ));
    	        	}  catch (Exception x) {
    	        		x.printStackTrace();
    	        		System.err.println(x.getMessage());
    	        	}
    	        
        	} catch (Exception x) {
        		x.printStackTrace();
        	}
        }
            
    }
 
    protected void setProperty(JComboBox box, Collection newProp) {
        if ((newProp != null) && (!newProp.equals(""))) {
                box.setSelectedIndex(-1);
                box.removeAllItems();
                Iterator i = ((Collection) newProp).iterator();
                while (i.hasNext()) {
                    Object o = i.next();
                    box.addItem(o.toString());                  
                }
        } else 
            box.removeAll();

    }
    public void chartMouseClicked(ChartMouseEvent event) {
        if (event.getEntity() instanceof XYItemEntity) {
            XYItemEntity xy = (XYItemEntity) event.getEntity();
            try {
                Subsets datasets = ((MoleculesXYZDataset)xy.getDataset()).getReaders();
                SelectedItem newSelected = new SelectedItem(
                            datasets,
                            datasets.get(xy.getSeriesIndex()),
                            xy.getItem());
                datasets.firePropertyChange(datasets.getName(), 
                        selectedItem, 
                        newSelected
                        );
                        /*
                firePropertyChange(datasets.getName(), 
                        selectedItem, 
                        newSelected
                        );
                        */
                setSelected(newSelected);
             } catch (Exception x) {
                
            }
        }    
        
    }
    public void chartMouseMoved(ChartMouseEvent event) {
        // TODO Auto-generated method stub
        
    }
    public void setSelected(SelectedItem selected) {
        if (selected == null) return;
        this.selectedItem.setDataset(selected.getDataset());
        this.selectedItem.setIndex(selected.getIndex());
        this.selectedItem.setSubset(selected.getSubset());
    }
    protected List sort(Collection values) {
        ArrayList l = new ArrayList();
        Iterator i = values.iterator();
        while (i.hasNext()) l.add(i.next());
        Collections.sort(l);
        return l;
    }
}
