/* DescriptorsAction.java
 * Author: Nina Jeliazkova
 * Date: Oct 1, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui.actions;

import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.openscience.cdk.aromaticity.HueckelAromaticityDetector;
import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.qsar.DescriptorValue;
import org.openscience.cdk.qsar.result.DoubleArrayResult;
import org.openscience.cdk.qsar.result.IDescriptorResult;

import ambit.data.HydrogenAdder;
import ambit.data.molecule.IAtomContainersList;
import ambit.io.Mopac7Reader;
import ambit.processors.descriptors.MopacShell;
import ambit.ui.UITools;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.IChemCastObject;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;
import chemCast.ui.ListPanel;

public class DescriptorsAction extends ToxmatchAction {

    /**
     * 
     */
    private static final long serialVersionUID = 7850294629412172555L;

    public DescriptorsAction(ChemCASTModule userData, JFrame mainFrame,
            int dataset) {
        this(userData, mainFrame, "Calculate descriptors", dataset);
    }

    public DescriptorsAction(ChemCASTModule userData, JFrame mainFrame,
            String name, int dataset) {
        this(userData, mainFrame, name, 
                UITools.createImageIcon("ambit/ui/images/calculator.png")
                ,dataset);
    }

    public DescriptorsAction(ChemCASTModule userData, JFrame mainFrame,
            String name, Icon icon, int dataset) {
        super(userData, mainFrame, name, icon, dataset);
    }

    @Override
    protected void processDataset(Subsets subsets) throws Exception {
        DatasetDef dataset = subsets.getParentDataset();
        if (dataset==null) {
            getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"No dataset available!");
            return;
        }
    	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Reading descriptors dictionary");
    	ChemCastDescriptorsList d = ChemCASTModule.readDescriptors();
    	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Selecting descriptors");
    	if (JOptionPane.showConfirmDialog(mainFrame,new ListPanel("Available descriptors", d),"Descriptors to calculate",JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE,null)
                == JOptionPane.OK_OPTION) {
    		
    		if (dataset != null)
	    		for (int j=0; j < d.getList().size();j++)
	    			if (d.get(j).isEnabled()) {
	    				subsets.firePropertyChange(IChemCastObject.p_properties, null,d.get(j).getTag());
	    				dataset.getProperties().getDescriptorProperties().put(d.get(j).getName(), d.get(j).getTag());
	    			}
    		
	        for (int i=0; i < subsets.size(); i++)
	            if (subsets.get(i).isEnabled())
	            	try {
	            		getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Progress,null,(i*100)/subsets.size());
	            		processDataset(dataset,d,subsets.get(i));
	            	} catch (SubsetException x) {
	            		x.printStackTrace();
	            	}
    	}     	
    }
    protected void processDataset(DatasetDef parentdataset, ChemCastDescriptorsList descriptors,Subset subset) throws SubsetException {
    	ChemCastDescriptorsList d = ChemCASTModule.readDescriptors();
    	IAtomContainersList data = subset.getData();
//        firePropertyChange("Processing", null,data.toString());
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Calculating descriptors for "+data.toString());
        HydrogenAdder ha = new HydrogenAdder();
        for (int i=0; i < data.getAtomContainerCount(); i++)
             try {
            	 getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Progress,null,new Double((100*i)/data.getAtomContainerCount()));
                    IAtomContainer a = data.getAtomContainer(i);
                    System.out.println(a.getAtomCount());
                    ha.addExplicitHydrogensToSatisfyValency(a);
                    System.out.println(a.getAtomCount());
                    HueckelAromaticityDetector.detectAromaticity(a);                    
                    for (int j=0; j < descriptors.getList().size();j++)
        	    		if (descriptors.get(j).isEnabled())
        	    			try {
        	    				getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Calculating "+descriptors.get(j)+ " for record #"+(i+1));
        	    				//Object v = data.getProperty(i,descriptors.get(j).getTag());
        	    				//if (v!=null) continue;
                                if (descriptors.get(j).getCalculator() instanceof MopacShell) {
                                    ((MopacShell)descriptors.get(j).getCalculator()).runMopac(a);
                                    for (int k=0; k < Mopac7Reader.parameters.length;k++) {
                                    	
                                        data.setProperty(i, Mopac7Reader.parameters[k],
                                            a.getProperty(Mopac7Reader.parameters[k]));    
                                    }    
                                } else {
            	    				DescriptorValue value = descriptors.get(j).calculate(a);
            	    				IDescriptorResult result = value.getValue();
                                    if (result instanceof DoubleArrayResult) {
                                        DoubleArrayResult dar = (DoubleArrayResult) result;
                                        for (int r =0; r < dar.size(); r++) {
                                            String dname = descriptors.get(j).getName()+"."+r;
                                            String dtag = descriptors.get(j).getTag()+"."+r;
                                            data.setProperty(i, dname, dar.get(r));
                                            parentdataset.getProperties().getDescriptorProperties().put(
                                                    dname, dtag);                                            
                                        }    
                                    }
                                    else data.setProperty(i, descriptors.get(j).getName(), result);
                                }
        	    			} catch (CDKException x) {
        	    				x.printStackTrace();
        	    				getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x.getMessage());
        	    				//x.printStackTrace();
        	    			}
             } catch (Exception x) {
            	 x.printStackTrace();
            	 getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x.getMessage());
                 firePropertyChange("Error", null,x);
             }
    }


}
