/* ImportDescriptorsAction.java
 * Author: Nina Jeliazkova
 * Date: Nov 7, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui.actions;

import java.io.File;
import java.io.FileInputStream;

import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.iterator.IIteratingChemObjectReader;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;

import ambit.data.molecule.PropertyTranslator;
import ambit.io.FileInputState;
import ambit.io.MolPropertiesIOSetting;
import ambit.io.MyIOUtilities;
import ambit.ui.UITools;
import ambit.ui.data.molecule.PropertyImporterPanel;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.data.BatchDescriptorImporter;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Subsets;

public class ImportDescriptorsAction extends ToxmatchAction {
    protected Object lookupProperty = CDKConstants.CASRN;
    protected PropertyTranslator properties = null;
    protected BatchDescriptorImporter batch;
    /**
     * 
     */
    private static final long serialVersionUID = -8947677070079272413L;

    public ImportDescriptorsAction(ChemCASTModule userData, JFrame mainFrame,
            int dataset) {
        this(userData, mainFrame,"Import", dataset);
    }

    public ImportDescriptorsAction(ChemCASTModule userData, JFrame mainFrame,
            String name, int dataset) {
        this(userData, mainFrame, name, UITools.createImageIcon("ambit/ui/images/batch.png"), dataset);
    }

    public ImportDescriptorsAction(ChemCASTModule userData, JFrame mainFrame,
            String name, Icon icon, int dataset) {
        super(userData, mainFrame, name, icon, dataset);
        properties = new PropertyTranslator();
    }

    @Override
    protected void processDataset(Subsets data) throws Exception {
        if (data.size()==0) return;
        
        /*
        DatasetDef dataset = null;
        for (int i=0;i<data.size();i++)
            if (data.get(i) instanceof DatasetDef) {
                dataset = (DatasetDef) data.get(i);
                break;
            }    
        if (dataset == null) throw new Exception("No data!");
        if (dataset.getData() == null) throw new Exception("No data!");
        */
        DatasetDef dataset = data.getParentDataset();
        if (dataset == null) throw new Exception("No data!");
        
        File file = MyIOUtilities.selectFile(mainFrame,null,
                getChemCastData().getDefaultDir(),
                FileInputState.extensions,FileInputState.extensionDescription,true);
        
        if (file == null) return;
        IIteratingChemObjectReader reader = FileInputState.getReader(new FileInputStream(file), file.getName());
        reader.addChemObjectIOListener(getReaderListener());
        
        batch = new BatchDescriptorImporter(reader,dataset.getData(),properties,lookupProperty);


        try {
            batch.start();
            
            properties.getProperties().clear();
            if (properties.getIdentifiers()!=null)
                properties.getIdentifiers().clear();
            dataset.addProperties(properties); 
            properties.clear();
        } catch (Exception x) {
            reader.close();
            throw new Exception(x);
        } 
        reader.close();
        
    }
    protected IReaderListener getReaderListener() {
        return new IReaderListener() {
          public void frameRead(ReaderEvent e) {
              getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
          }
          public void processIOSettingQuestion(IOSetting setting) {
              if (setting instanceof MolPropertiesIOSetting) {
                  if (properties.isEmpty()) {
                      PropertyImporterPanel panel = new PropertyImporterPanel(((MolPropertiesIOSetting)setting).getProperties(),lookupProperty);
                      if (JOptionPane.showConfirmDialog(mainFrame, 
                          panel
                      ,"Select descriptors to be imported",JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE,null
                      ) == JOptionPane.OK_OPTION) {
                          
                          batch.updateProperties(((MolPropertiesIOSetting)setting).getProperties(),panel.getLookupProperty());
                      }
                  }
              }  
          }
        };
    }
}
