/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.ui.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.openscience.cdk.interfaces.IAtomContainer;

import com.nexes.wizard.WizardPanelDescriptor;

import ambit.data.molecule.IAtomContainersList;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.IAmbitResult;
import ambit.similarity.ISimilarityProcessor;
import ambit.ui.UITools;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.SimilarityDef;
import chemCast.similarity.data.SimilarityModel;
import chemCast.similarity.data.SimilaritySubsets;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;
import chemCast.ui.ListPanel;
import chemCast.ui.RadioBoxPanel;
import chemCast.weka.ISubsetsModel;

public class SimilarityCalculationAction extends ToxmatchAction {
    /**
     * 
     */
    private static final long serialVersionUID = -3192948829589060707L;
    protected int processorIndex = ChemCASTModule.DATA_TRAINING;
    /**
     * 
     * @param userData
     * @param mainFrame
     * @param dataset  ChemCASTModule.DATA_TRAINING or ChemCASTModule.DATA_TEST
     * @param processor ChemCASTModule.DATA_TRAINING or ChemCASTModule.DATA_TEST
     */
	public SimilarityCalculationAction(ChemCASTModule userData, JFrame mainFrame,
			int dataset, int processor) {
		this(userData, mainFrame,"Similarity", dataset, processor);
		
	}
	/**
     * 
     * @param userData
     * @param mainFrame
     * @param name
     * @param dataset  ChemCASTModule.DATA_TRAINING or ChemCASTModule.DATA_TEST
     * @param processor ChemCASTModule.DATA_TRAINING or ChemCASTModule.DATA_TEST
	 */
	public SimilarityCalculationAction(ChemCASTModule userData, JFrame mainFrame,
			String name, int dataset,int processor) {
		this(userData, mainFrame, name,  UITools.createImageIcon("ambit/ui/images/run.png"),
                dataset,processor);
		
	}
	/**
     * 
     * @param userData
     * @param mainFrame
     * @param name
     * @param icon
     * @param dataset  ChemCASTModule.DATA_TRAINING or ChemCASTModule.DATA_TEST
     * @param processor ChemCASTModule.DATA_TRAINING or ChemCASTModule.DATA_TEST
	 */
	public SimilarityCalculationAction(ChemCASTModule userData, JFrame mainFrame,
			String name, Icon icon, int dataset,int processor) {
		super(userData, mainFrame, name, icon,dataset);
        this.processorIndex = processor;
		
	}

    public void process(SimilarityModel model,Subsets subsets)    throws Exception  {
        getChemCastData().getPropertyChangeSupport().firePropertyChange("Processor",null,(ISimilarityProcessor)null);
        ISimilarityProcessor processor = model.getProcessor();
        if (!processor.isPredicting()) {
        	if (processorIndex != dataset) throw new Exception(processor.toString() + " not build on training set yet!");
            if (processor instanceof ISubsetsModel)
                buildModel((ISubsetsModel)processor,subsets);
            else
                build(processor,getChemCastData().getDataset(processorIndex));
            processor.close();
        }   
        if (processor instanceof ISubsetsModel)
            processModel((ISubsetsModel)processor,subsets);
        else {
            for (int i=0; i < subsets.size(); i++)
            	if (subsets.get(i).isEnabled())
            		process(processor,subsets.get(i));
            IAmbitResult result = processor.getResult();
            if (result != null) result.setTitle(subsets.toString());
        }    
        String[] properties = model.getProcessor().getProperties();
        DatasetDef d = subsets.getParentDataset();
        if (d != null)
            for (int i=0; i < ChemCASTModule.DATA_COUNT;i++) {
            	DatasetDef pdata = getChemCastData().getDataset(i).getParentDataset();
            	if (pdata != null)
            		for (int j=0; j < properties.length;j++)
            			pdata.getProperties().addProperty("similarity",properties[j], properties[j]);
            }	        
        processor.close();
        getChemCastData().getPropertyChangeSupport().firePropertyChange("Processor",null,processor);
    }
    protected void buildModel(ISubsetsModel processor,Subsets subsets)   throws Exception  {
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,processor.toString());
        processor.setDatasets(getChemCastData().getDataset(processorIndex));
        processor.build();
    }
    protected void processModel(ISubsetsModel processor,Subsets subsets)   throws Exception  {
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,processor.toString());
        processor.setDatasets(subsets);
        processor.predict();
    }
    protected void build(ISimilarityProcessor processor,Subsets subsets)   throws SubsetException  {
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,processor.toString());
        for (int i=0; i < subsets.size(); i++)
            if (subsets.get(i).isEnabled())
                build(processor,subsets.get(i));
    }

    protected void process(ISimilarityProcessor processor, Subset subset) throws SubsetException  {
        IAtomContainersList data = subset.getData();
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Prediction phase: " + processor.toString() + " " + data.toString());
        
        String prop = processor.getTrainingPrefix() + "." + processor.getSimilarityProperty();
        for (int i=0; i < data.getAtomContainerCount(); i++)
             try {
            	 getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/data.getAtomContainerCount()));
                    IAtomContainer a = data.getAtomContainer(i);
                    if (a != null) {
                        processor.process(a);
                        data.setProperty(i,prop,
                                a.getProperty(prop));
                    } else System.err.println("index "+i);
             } catch (Exception x) {
                 
                 getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x);
             }
                 
    }
    protected void build(ISimilarityProcessor processor, Subset subset)   throws SubsetException  {
            IAtomContainersList data = subset.getData();
            getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Build phase: " + processor.toString() + " " + data.toString());
            for (int i=0; i < data.getAtomContainerCount(); i++)
                try {
                	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/data.getAtomContainerCount()));
                    IAtomContainer a = data.getAtomContainer(i);
                    processor.process(a);
                    if (a.getProperty(AmbitCONSTANTS.Fingerprint) != null)
                        data.setProperty(i, 
                                AmbitCONSTANTS.Fingerprint, 
                                a.getProperty(AmbitCONSTANTS.Fingerprint));

                } catch (Exception x) {
                	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x);
                    
                }
          
    }
    @Override
    protected void processDataset(Subsets data) throws Exception {
    	SimilarityModel selectedSimilarityModel = null;
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(false));
        boolean proceed = false;
        if (data.getList().size()==0) return;
        if (processorIndex != dataset) {
            //calculating similarity to the other set
            if (getChemCastData().getDataset(processorIndex).getProcessors().getList().size()==0) return;
            RadioBoxPanel similarityPanel = new RadioBoxPanel("Similarity methods",
                    getChemCastData().getDataset(processorIndex).getProcessors().getList(),0){
                @Override
                public void selectObject(ActionEvent e, Object object) {
                    if (object instanceof SimilarityDef) {
                        //descriptor1.setNextId(WizardPanelDescriptor.FINISH);
                    }
                }
            };
            if (JOptionPane.showConfirmDialog(mainFrame,similarityPanel,"Available similarity measures to the "+ChemCASTModule.datasetName[processorIndex],
                    JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE,null)==JOptionPane.OK_OPTION) {
            	selectedSimilarityModel = getChemCastData().getDataset(processorIndex).getProcessors().get(similarityPanel.getSelectedIndex());
                //((SimilaritySubsets) data).setSimilarity(m);
                proceed = (selectedSimilarityModel != null);
            }  else proceed = false;

        } else {
            proceed = getChemCastData().selectSimilarity(mainFrame,getChemCastData().getDataset(processorIndex));
            if (proceed) {
            	selectedSimilarityModel = getChemCastData().getSelectedProcessor(processorIndex);
            	proceed = (selectedSimilarityModel != null);
            }	
        }
        
        if (proceed) {
            getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
            if (selectedSimilarityModel == null) 
            	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"No similarity method!");
            else
            try {
                process(selectedSimilarityModel,data);
                
  

            } catch (SubsetException x) {
            	getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x);
                JOptionPane.showMessageDialog(mainFrame, x.toString(),"Error",JOptionPane.ERROR_MESSAGE);
            }
        }
        
    }
    protected void firePropertyChanged(String propertyName, Object oldValue, Object newValue) {
        getChemCastData().getPropertyChangeSupport().firePropertyChange(propertyName, oldValue, newValue);
    }
	
}


