/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import weka.core.DistanceFunction;
import weka.core.Instance;
import weka.core.Instances;

/**
 * 1-(Cosine similarity).<br>     
 * Cosine similarity is Cab=Zab/Math.sqrt(Zaa*Zbb). <br>
 * or <br> 
 * Sum(Xsj*Xtj)/Math.sqrt[Sum(Xsj*Xsj)^2+Sum(Xtj*Xtj)^2]
 * This class calculates dissimilarity (to be compatible with {@link DistanceFunction}
 * int the form of 1-Cab
 * 
 * 
 * @author Nina Jeliazkova
 *
 */
public class CosineSimilarity extends AbstractSimilarity {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6749620577373296394L;
	public CosineSimilarity() {
		super();

	}

	public CosineSimilarity(Instances data) {
		super(data);
	}
    public float getDistance(Instance a, Instance b) throws Exception {
        return (float)distance(a, b,Float.MAX_VALUE);
    }

    public float getNativeComparison(Instance a, Instance b) throws Exception {
        double d = zAA(a)*zAA(b);
        double zab = zAB(a,b);
        if (d != 0)
            return  (float) (zab/Math.sqrt(d));
        else
            return 0;         
    }    
	@Override
	public double distance(Instance a, Instance b) throws Exception {
		return distance(a,b,Double.MAX_VALUE,false);
	}
	@Override
	public double distance(Instance a, Instance b, double cutoff) {
		return distance(a, b, cutoff, false);
	}
	@Override
	public double distance(Instance a, Instance b, double cutoff, boolean print) {
		double d = zAA(a)*zAA(b);
        double zab = zAB(a,b);
		if (d != 0)
		    return  similarity2distance(zab/Math.sqrt(d));
		else
            return cutoff; //infinite
        
        
	}	
	@Override
	/**
	 * Does nothing
	 */
	public void postProcessDistances(double[] distances) {
        /*
	    for(int i=0; i<distances.length; i++)
	    	if (distances[i]==0)
	    		distances[i] = Double.MAX_VALUE;
	    	else	
	    		distances[i] = 1/distances[i];
                */
	}
    @Override
    public String toString() {
        return "Cossine coefficient = Sum(Xsj*Xtj)/Math.sqrt[Sum(Xsj*Xsj)^2+Sum(Xtj*Xtj)^2]";
    }


}


