/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.test;

import junit.framework.TestCase;

import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.DefaultChemObjectBuilder;
import org.openscience.cdk.Molecule;
import org.openscience.cdk.aromaticity.HueckelAromaticityDetector;
import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.isomorphism.UniversalIsomorphismTester;
import org.openscience.cdk.smiles.SmilesGenerator;
import org.openscience.cdk.smiles.SmilesParser;
import org.openscience.cdk.templates.MoleculeFactory;

/**
 * TODO add description
 * @author Nina Jeliazkova
 * <b>Modified</b> 2005-9-20
 */
public class AromaticityTest extends TestCase {

	public static void main(String[] args) {
		junit.textui.TestRunner.run(AromaticityTest.class);
	}
	public void aromaticRoundTrip(String smiles) {
		SmilesParser p = new SmilesParser();
		try {
			Molecule m = p.parseSmiles(smiles);
			aromaticRoundTrip(m);
		} catch (CDKException x) {
			
			fail();
		}		
	}
	
	public void aromaticRoundTrip(Molecule m) {
		SmilesParser p = new SmilesParser();
		SmilesGenerator sg = new SmilesGenerator(DefaultChemObjectBuilder.getInstance());
		try {
			
			boolean b = HueckelAromaticityDetector.detectAromaticity(m,true);
			/*
			for (int i=0; i<m.getBondCount();i++) {
				System.out.println("order "+m.getBondAt(i).getOrder());
				if (m.getBondAt(i).getFlag(CDKConstants.ISAROMATIC))
				System.out.println("\taromatic");
			}
			*/
			assertTrue(b);
				
			String s = sg.createSMILES(m);
			System.out.println(s);
		
			Molecule m1 = p.parseSmiles(s);
			
			b= HueckelAromaticityDetector.detectAromaticity(m1,true);
			
			for (int i=0; i<m1.getBondCount();i++) {
				System.out.println("order "+m1.getBondAt(i).getOrder());
				if (m1.getBondAt(i).getFlag(CDKConstants.ISAROMATIC))
				System.out.println("\taromatic");
			}
			
			assertTrue(b);
			
			assertTrue(UniversalIsomorphismTester.isIsomorph(m,m1));
			
		} catch (CDKException x) {
			
			fail();
		}		
	}

	public void test1() {
		//this fails, but should not 
		aromaticRoundTrip("C1=CC=CC=C1C2=C(C(=O)OCC)OC=C2");
	}
	public void testBenzene() {
		//this is ok
		aromaticRoundTrip("C1=CC=CC=C1");
	}
	
	public void test2() {
		//this fails, but should not 
		aromaticRoundTrip("c1ccco1");
	}
	public void test3() {
		aromaticRoundTrip("C1=CC=CC=C1CCCC1=CC=C(CCC)C=C1");
	}	
	public void test4() {
		//this fails, but should not 
		aromaticRoundTrip(MoleculeFactory.makeIndole());
	
	}

	
}

