/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package toxTree.ui.tree.rules;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import joelib.smarts.JOESmartsPattern;
import toxTree.core.IDecisionRuleEditor;
import toxTree.tree.rules.smarts.RuleSMARTSubstructure;
import toxTree.tree.rules.smarts.SMARTSException;
import toxTree.ui.HashtableModel;

/**
 * Editor for {@link RuleSMARTSubstructure}.
 * @author Nina Jeliazkova
 *
 */
public class SMARTSRuleEditor extends RuleEditor implements
		IDecisionRuleEditor {
	/**
	 * 
	 */
	private static final long serialVersionUID = -8216887948144449098L;
	protected HashtableModel tableModel;
	protected JCheckBox allBox = null;
	protected JTable table;
	protected int selectedIndex = -1;

	public SMARTSRuleEditor(RuleSMARTSubstructure object) {
		super(object);
		setRule((RuleSMARTSubstructure)object);
		
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab("SMARTS",createSmartsPane());
		tabbedPane.addTab("Explanation",explanationPane);
		add(tabbedPane,BorderLayout.CENTER);
	}

	public Component createSmartsPane() {
		JPanel p = new JPanel(new BorderLayout());
		tableModel = new HashtableModel(((RuleSMARTSubstructure)rule).getSmartsPatterns(),true) {
			@Override
			public Object getValueAt(int row, int col) {
				Object o = super.getValueAt(row, col);
				switch(col) {
				case 1: {
					if (o instanceof JOESmartsPattern)
						return ((JOESmartsPattern)o).getSMARTS();
					else 	
						return o.toString();
				}
				}
				return o;
			};
		};
		table = new JTable(tableModel);
		table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting()) return;
				else {
					selectedIndex = ((ListSelectionModel) e.getSource()).getMinSelectionIndex();
				}
			}
		});
		p.add(new JScrollPane(table),BorderLayout.CENTER);
		JPanel toolbar = new JPanel();
		JButton b = new JButton(new AbstractAction("Add SMARTS"){
			public void actionPerformed(ActionEvent arg0) {
				editSmarts();
			}
		});
		b.setToolTipText("Add SMARTS pattern");
		toolbar.add(b);
		
		b = new JButton(new AbstractAction("Edit") {
			public void actionPerformed(ActionEvent arg0) {
				if (selectedIndex == -1) return;
				editSmarts(table.getValueAt(selectedIndex, 0).toString(), table.getValueAt(selectedIndex, 1).toString());
			};
		});
		b.setToolTipText("Edit SMARTS pattern");
		toolbar.add(b);		
		
		b = new JButton(new AbstractAction("Delete") {
			public void actionPerformed(ActionEvent arg0) {
				deleteSmarts();
			};
		});
		b.setToolTipText("Delete SMARTS pattern");
		toolbar.add(b);		
		
		allBox = new JCheckBox("All");
		allBox.addItemListener(new ItemListener() {
			public void itemStateChanged(java.awt.event.ItemEvent e) {
				if (e.getStateChange() == ItemEvent.DESELECTED) {
					((RuleSMARTSubstructure)rule).setContainsAllSubstructures(false);
					allBox.setText("Any");
				} else {
					allBox.setText("All");
					((RuleSMARTSubstructure)rule).setContainsAllSubstructures(true);
				}
	
			};
		});
		allBox.setToolTipText("Returns true if the query structure contains all or any of the substructures defined by SMARTS.");
		allBox.setSelected(((RuleSMARTSubstructure)rule).containsAllSubstructures());
		allBox.addFocusListener(this);
		
		toolbar.add(allBox);
		
		p.add(toolbar,BorderLayout.NORTH);
		return p;
	}
	protected void launchErrorDialog(Exception x) {
		JOptionPane.showMessageDialog(this, x.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
	
	}
	public void deleteSmarts() {
		if (selectedIndex == -1) return;
		
		if (JOptionPane.showConfirmDialog(this,
				table.getValueAt(selectedIndex,0) + "\n" + table.getValueAt(selectedIndex,1)  
				,"Delete SMARTS", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION)
			try {
				((RuleSMARTSubstructure)rule).deleteSubstructure(table.getValueAt(selectedIndex,0).toString());
				tableModel.setTable(((RuleSMARTSubstructure)rule).getSmartsPatterns());
			} catch (Exception x) {
				launchErrorDialog(x);
			}
	}
	public void editSmarts() {
		editSmarts("","");
	}
	public void editSmarts(String title, String smarts) {
		SmartsPanelEditor smartsEditor = new SmartsPanelEditor(title,smarts);
		if (JOptionPane.showConfirmDialog(this, smartsEditor,"Enter SMARTS",JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE) == JOptionPane.OK_OPTION) {
			smarts = smartsEditor.getSMARTS();
			title = smartsEditor.getTitle();
			if (smarts != null)
				try {
					((RuleSMARTSubstructure)rule).addSubstructure(title,smarts);
					tableModel.setTable(((RuleSMARTSubstructure)rule).getSmartsPatterns());
				} catch (SMARTSException x) {
					launchErrorDialog(x);
				}
		}	
		smartsEditor = null;
	}
	
	public SMARTSRuleEditor() {
		this(null);

	}


}


